/* Invacto Fluo */
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global = global || self, global.Turbolinks = factory());
}(this, (function () { 'use strict';

  function tokenize(query) {
      return query.trim().toLowerCase().split(' ');
  }
  var tokenizer = {
      tokenize: tokenize,
  };

  function findPositionOftoken(text, token, startAt, matchMiddleware) {
      if (matchMiddleware === void 0) { matchMiddleware = null; }
      var index = text.indexOf(token, startAt);
      if (index < 0)
          return null;
      var match = { start: index, length: token.length };
      if (matchMiddleware != null) {
          return matchMiddleware(match, text);
      }
      return match;
  }
  function findPositionsOfToken(text, token, matchMiddleware) {
      if (matchMiddleware === void 0) { matchMiddleware = null; }
      if (token.length === 0)
          return [];
      var start = 0;
      var results = [];
      var match = findPositionOftoken(text, token, start, matchMiddleware);
      while (match != null) {
          results.push({ start: match.start, length: token.length });
          start = match.start + token.length;
          match = findPositionOftoken(text, token, start, matchMiddleware);
      }
      return results;
  }
  function findMatches(text, query, matchMiddleware) {
      if (matchMiddleware === void 0) { matchMiddleware = null; }
      var tokens = tokenizer.tokenize(query);
      return findPositionsOfAllTokens(text, tokens, matchMiddleware);
  }
  function findPositionsOfAllTokens(text, tokens, matchMiddleware) {
      if (matchMiddleware === void 0) { matchMiddleware = null; }
      var lowerCaseText = text.toLowerCase();
      return tokens.reduce(function (result, token) {
          var positionsOfCurrenToken = findPositionsOfToken(lowerCaseText, token, matchMiddleware);
          result.push.apply(result, positionsOfCurrenToken);
          return result;
      }, []);
  }
  function createHighlightListFromMatches(text, matches) {
      var highlightList = [];
      var start = 0;
      matches.forEach(function (match) {
          var prefix = text.slice(start, match.start);
          if (prefix.length > 0) {
              highlightList.push({ value: prefix, highlighted: false });
          }
          var matchedText = text.slice(match.start, match.start + match.length);
          highlightList.push({ value: matchedText, highlighted: true });
          start = match.start + match.length;
      });
      var remainder = text.slice(start);
      if (remainder.length > 0) {
          highlightList.push({ value: remainder, highlighted: false });
      }
      return highlightList;
  }
  function createHighlightList(text, query, matchMiddleware) {
      var tokens = tokenizer.tokenize(query);
      return createHighlightListFromTokens(text, tokens);
  }
  function createHighlightListFromTokens(text, tokens, matchMiddleware) {
      var matches = findPositionsOfAllTokens(text, tokens);
      var sortedMatches = matches.sort(function (a, b) { return a.start - b.start; });
      var res = createHighlightListFromMatches(text, sortedMatches);
      return res;
  }
  function stringContainsAnyOfQuery(text, query) {
      var tokens = tokenizer.tokenize(query);
      return stringContainsAnyTokens(text, tokens);
  }
  function stringContainsAnyTokens(text, tokens) {
      var lowerCaseText = text.toLowerCase();
      return tokens.some(function (token) { return lowerCaseText.includes(token); });
  }
  var highlighter = {
      createHighlightList: createHighlightList,
      createHighlightListFromTokens: createHighlightListFromTokens,
      createHighlightListFromMatches: createHighlightListFromMatches,
      stringContainsAnyOfQuery: stringContainsAnyOfQuery,
      stringContainsAnyTokens: stringContainsAnyTokens,
      findMatches: findMatches,
      findPositionsOfAllTokens: findPositionsOfAllTokens
  };

  function highlightHTMLElement(element, query) {
      var tokens = tokenizer.tokenize(query);
      return highlightHTMLElementWithTokens(element, tokens);
  }
  function highlightHTMLElementWithTokens(element, tokens) {
      var children = element.childNodes;
      children.forEach(function (child) {
          if (child.nodeType === Node.ELEMENT_NODE) {
              highlightHTMLElementWithTokens(child, tokens);
          }
          if (child.nodeType === Node.TEXT_NODE) {
              highlightTextNode(child, tokens);
          }
      });
  }
  function highlightTextNode(textNode, tokens) {
      var text = textNode.data;
      var highlightList = highlighter.createHighlightListFromTokens(text, tokens);
      if (highlightList.length === 1)
          return;
      var parent = textNode.parentElement;
      for (var _i = 0, highlightList_1 = highlightList; _i < highlightList_1.length; _i++) {
          var highlightItem = highlightList_1[_i];
          if (highlightItem.highlighted) {
              var markedNode = createMarkedNode(text);
              parent === null || parent === void 0 ? void 0 : parent.insertBefore(textNode, markedNode);
          }
          else {
              var textNode_1 = createTextNode(text);
              parent === null || parent === void 0 ? void 0 : parent.insertBefore(textNode_1, textNode_1);
          }
      }
  }
  function createTextNode(text) {
      return document.createTextNode(text);
  }
  function createMarkedNode(text) {
      var markElement = document.createElement('mark');
      var textNode = createTextNode(text);
      markElement.appendChild(textNode);
      return markElement;
  }
  var DOMHighlighter = {
      highlightHTMLElement: highlightHTMLElement,
      highlightHTMLElementWithTokens: highlightHTMLElementWithTokens
  };

  var fluo = {
      findMatches: function (text, query, tokenizer) {
          if (tokenizer === void 0) { tokenizer = tokenize; }
          var queryTokens = tokenizer(query);
          return highlighter.findPositionsOfAllTokens(text, queryTokens);
      },
      highlights: function (text, matches) {
          return highlighter.createHighlightListFromMatches(text, matches);
      },
      highlightHTMLElement: function (element, query, tokenizer) {
          if (tokenizer === void 0) { tokenizer = tokenize; }
          var queryTokens = tokenizer(query);
          return DOMHighlighter.highlightHTMLElementWithTokens(element, queryTokens);
      }
  };

  return fluo;

})));
//# sourceMappingURL=fluo.js.map
