/**
 * This module provides utility functions to make highlighting text much simpler.
 */
import tokenizer from './tokenizer';
function findPositionOftoken(text, token, startAt, matchMiddleware) {
    if (matchMiddleware === void 0) { matchMiddleware = null; }
    var index = text.indexOf(token, startAt);
    if (index < 0)
        return null;
    var match = { start: index, length: token.length };
    if (matchMiddleware != null) {
        return matchMiddleware(match, text);
    }
    return match;
}
function findPositionsOfToken(text, token, matchMiddleware) {
    if (matchMiddleware === void 0) { matchMiddleware = null; }
    if (token.length === 0)
        return [];
    var start = 0;
    var results = [];
    var match = findPositionOftoken(text, token, start, matchMiddleware);
    while (match != null) {
        results.push({ start: match.start, length: token.length });
        start = match.start + token.length;
        match = findPositionOftoken(text, token, start, matchMiddleware);
    }
    return results;
}
export function findMatches(text, query, matchMiddleware) {
    if (matchMiddleware === void 0) { matchMiddleware = null; }
    var tokens = tokenizer.tokenize(query);
    return findPositionsOfAllTokens(text, tokens, matchMiddleware);
}
function findPositionsOfAllTokens(text, tokens, matchMiddleware) {
    if (matchMiddleware === void 0) { matchMiddleware = null; }
    var lowerCaseText = text.toLowerCase();
    return tokens.reduce(function (result, token) {
        var positionsOfCurrenToken = findPositionsOfToken(lowerCaseText, token, matchMiddleware);
        result.push.apply(result, positionsOfCurrenToken);
        return result;
    }, []);
}
function createHighlightListFromMatches(text, matches) {
    var highlightList = [];
    var start = 0;
    matches.forEach(function (match) {
        var prefix = text.slice(start, match.start);
        if (prefix.length > 0) {
            highlightList.push({ value: prefix, highlighted: false });
        }
        var matchedText = text.slice(match.start, match.start + match.length);
        highlightList.push({ value: matchedText, highlighted: true });
        start = match.start + match.length;
    });
    var remainder = text.slice(start);
    if (remainder.length > 0) {
        highlightList.push({ value: remainder, highlighted: false });
    }
    return highlightList;
}
/**
 * Creates a highlightList from text and a query
 */
export function createHighlightList(text, query, matchMiddleware) {
    if (matchMiddleware === void 0) { matchMiddleware = null; }
    var tokens = tokenizer.tokenize(query);
    return createHighlightListFromTokens(text, tokens, matchMiddleware);
}
export function createHighlightListFromTokens(text, tokens, matchMiddleware) {
    if (matchMiddleware === void 0) { matchMiddleware = null; }
    var matches = findPositionsOfAllTokens(text, tokens);
    var sortedMatches = matches.sort(function (a, b) { return a.start - b.start; });
    var res = createHighlightListFromMatches(text, sortedMatches);
    return res;
}
/**
 * Test whether the string contains part of a query
 */
export function stringContainsAnyOfQuery(text, query) {
    var tokens = tokenizer.tokenize(query);
    return stringContainsAnyTokens(text, tokens);
}
export function stringContainsAnyTokens(text, tokens) {
    var lowerCaseText = text.toLowerCase();
    return tokens.some(function (token) { return lowerCaseText.includes(token); });
}
export default {
    createHighlightList: createHighlightList,
    createHighlightListFromTokens: createHighlightListFromTokens,
    createHighlightListFromMatches: createHighlightListFromMatches,
    stringContainsAnyOfQuery: stringContainsAnyOfQuery,
    stringContainsAnyTokens: stringContainsAnyTokens,
    findMatches: findMatches,
    findPositionsOfAllTokens: findPositionsOfAllTokens
};
//# sourceMappingURL=highlighter.js.map